##########################################################################
# Supplementary materials for
# Bakk, Z. and Kuha, J. (2017). Two-step estimation of models between latent
# classes and external variables. Psychometrika.
#
# Code for two-step estimation in the two applied examples in the paper.
# In both the logic of the calculations is the following:
#
# - R and Latent Gold need to be installed. Latent Gold Version 5.0 was used here
# - The procedure is initiated and completed in R, which calls Latent Gold several times
# - Several Latent Gold command files are prepared in advance, corresponding to
#	the different steps of the estimation. Variable names, model formulas etc. are
#	hard-coded in these files. What R adds to the command files before step 2 are
#	the values of the relevant parameter estimates from the previous step, where these are
#	used as fixed values or starting values for the next step.
# - The data files are in SPSS format, which is used by Latent Gold. 	
#
# NOTE: 
# For the one-step, step one  and three-step estimators, code for obtaining these in Latent Gold 
# is provided in the supplementary materials. These can be run directly in Latent Gold.

library(brew)

# Function for running Latent Gold syntax files from R.
# Change the location of the Latent Gold executable to what applies to you.
# 
run.template <- function(template.path, envir, temp.filename.base) {
    temp.filename <- paste(temp.filename.base, ".lgs", sep="")

    template <- file(template.path, 'r')
    brew(template, output=temp.filename, envir=envir) # writes to file
    close(template)

# Change this to the location of Latent Gold executable in your system 	
    shell(sprintf('"C:\\Users\\bakkz\\Documents\\leiden2step\\LatentGOLD5.0\\lg50.exe" %s', temp.filename))
}

# Change to workspace where the Latent Gold files for these examples are saved

setwd('C:\\Documents\\Supplementarymaterial_Bakk_Kuha_2017') 


#############################################
# EXAMPLE IN SECTION 4.1: TOLERANCE TOWARD NONCONFORMITY

# Step 1. Estimate and save step-1 model parameters for use in step 2

envir <- new.env()
run.template("step1_mccutcheon.brew", envir=envir, temp.filename.base="step1_mccutcheon")


# Step 2a: Estimate step-2 model with step 1 parameters fixed at their estimates 

parsstep1 <-as.matrix(read.table('parameters_mccutcheon1.dat', sep=" ", dec=","))
bpars <- (parsstep1[-c(1:3)])   # Exclude parameters of the model for the probabilities of the latent classes
envir <- new.env()
assign("bpars", bpars, envir=envir) 
run.template("step2a_mccutcheon.brew", envir=envir, temp.filename.base="step2a_mccutcheon")

## Calculation of correct standard errors

#### Step-1 variance matrix (Sigma_11 in the article)
Step1matrices <-read.table('hesstep1_mccutcheon.dat', header=F) # read in the LG output with the relevant matrices
Step1Var <- as.matrix(Step1matrices[24:46,]) # SE estimator of step1 
Sigma11 <- as.matrix(Step1Var[4:23,4:23]) # leave out parameters of the model for the latent class probabilities 

#### Step 2b: estimation with 0 iterations, starting with the estimated parameter values, to obtain standard error correction terms. 
pars2 <-as.matrix(read.table('step2Apars_mccutcheon.dat', sep=" ", dec=","), col.names = FALSE)
apars <-c(pars2,bpars)
envir <- new.env()
assign("apars", apars, envir=envir)
run.template("step2b_mccutcheon.brew", envir=envir, temp.filename.base="step2b_mccutcheon")
Step2bmatrices <-read.table('step2Bhes_mccutcheon.dat' , header=F)

I22 <- as.matrix(Step2bmatrices [1:12,1:12]) # I_22 in the article  
I21 <- as.matrix(Step2bmatrices [1:12,13:32]) # I_21(=I_12') in the article 

#correction term for step2
V1 <- solve(I22)%*%I21%*%Sigma11%*%t(I21)%*%solve(I22) 
V1 <- V1*(2668/2689) # Correction for different samples sizes in the two steps

## Read in step-2 uncorrected variance  
sigma2 <-read.table("step2Ahes_mccutcheon.dat", skip=1, sep = "", 
                    dec = ".")
V2 <- as.matrix(sigma2[12:23,]) # This could also be obtained as solve(I22)

V <- V2 + V1
SEcor <- round(sqrt(diag(V)), digits=4) #final corrected SE. 
SEnaive <- round(sqrt(diag(V2)) , digits=4) ##naive SE estimates 
Estimate <- round(as.numeric(pars2), digits=4)
### display point estimate sof step2 model and corrected and naive SE. 
Step2estimates <-data.frame((cbind(Estimate,SEcor,SEnaive)))
row.names(Step2estimates) <-c('intercept1','intercept2','intercept3', 
                               'Cohort_IntolR', 'Cohort_IntolL','Cohort_Intol',
                              'HIghSchool_IntolR', 'HIghSchool_IntolL', 'HIghSchool_Intol', 
                              'College_IntolR', 'College_IntolL','College_Intol')
print(Step2estimates)

#############################################
# EXAMPLE IN SECTION 4.2: PSYCHOLOGICAL CONTRACT TYPES

# Step 1. Estimate and save step-1 model parameters for use in step 2

envir <- new.env()
run.template("step1_psycon.brew", envir=envir, temp.filename.base="step1_psycon")

# Step 2a: Estimate step-2 model with step 1 parameters fixed at their estimates 

parspsy <-as.matrix(read.table('parameters_psycon.dat', sep=" ", dec=","))
envir <- new.env()
assign("parspsy", parspsy, envir=envir) 
run.template("step2a_psycon.brew", envir=envir, temp.filename.base="step2a_psycon")

## Calculation of correct standard errors

#### Step-1 variance matrix (Sigma_11 in the article)
Step1matrices_psy <-read.table('hesstep1_psycon.dat', header=F) #read in the LG output with the relevant matrices
Sigma11 <- as.matrix(Step1matrices_psy[36:70,]) ### SE estimator of step1 

#### Step 2b: estimation with 0 iterations, starting with the estimated parameter values, to obtain standard error correction terms. 

pars2psycon <-as.matrix(read.table('step2Apars_psycon.dat', sep=" ", dec=","), col.names = FALSE)
pars2bpsycon <-c(parspsy, pars2psycon)
envir <- new.env()
assign("pars2bpsycon", pars2bpsycon, envir=envir)
run.template("step2B_psycon.brew", envir=envir, temp.filename.base="step2B_psycon")
Step2bmatrices <-read.table('step2Bhes_psycon.dat' , header=F)

I22 <- as.matrix(Step2bmatrices [36:39,36:39]) # I_22 in the article  
I21 <- as.matrix(Step2bmatrices [36:39,1:35]) # I_21(=I_12') in the article  

#correction term for step2
V1 <- solve(I22)%*%I21%*%Sigma11%*%t(I21)%*%solve(I22) 
V1 <- V1*(1427/1431) # Correction for different samples sizes in the two steps

# Read in step2 uncorrected variance  
sigma2 <-read.table("step2Ahes_psycon.dat", skip=1, sep = "", dec = ".")
V2 <-as.matrix(sigma2[5:8,1:4]) # This could also be obtained as solve(I22)
V <- V2 + V1[1:4,1:4]

SEcor <- round(sqrt(diag(V)), digits=4) #final corrected SE.  
SEnaive <- round(sqrt(diag(V2)), digits=4)
estimates<-round(as.numeric(pars2psycon[1:4]), digits=4)
Psyconstep2 <- cbind(estimates, SEcor,SEnaive)
row.names(Psyconstep2) <- c('intercept', 'over-obligation', 'under-obligation', 'mutual_low')
print(Psyconstep2)
############################################################################
